--[[

	Martin Karer / Sezz, 2010-2011
	Action Bar Styling, based on rActionBarStyler (by roth, 2009)

	Use /bags to show/hide the bags frame!
	
	TODO: PossessBar (replace totems/stances(shapeshift)), Vehicle Exit/Roration Buttons

	------------------------------
	
	Changelog:
	2011-07-03:
		[+] Added Dungeon Journal and Raid micro buttons

	2010-09-13:
		[:] Cataclysm Update
		[*] Fixed some minor errors
		[+] Added the new MicroMenu buttons

	2010-07-14:
		[*] Fixed micro menu disappearing instantly after clicking a button when hovering is enabled

	2010-06-28:
		[*] Fixed memory usage...

	2010-06-26/2010-06-27:
		[*] rActionBarStyler rewritten

--]]

-- Locals/Constants
local _G = getfenv(0);
local abs = math.abs;
-- Optional s:Core Support
local UI_BUTTON_PADDING = UI_BUTTON_PADDING or 2;
local UI_BUTTON_SIZE = UI_BUTTON_SIZE or 30;

---------------------------------------------------
-- Settings
---------------------------------------------------

-- Debugging -> Displays ugly backdrops :)
local enableDebug = 0;
local enableDebugBackDrop = {bgFile = "Interface/Tooltips/UI-Tooltip-Background", edgeFile = "", tile = true, tileSize = 16, edgeSize = 16, insets = { left = 0, right = 0, top = 0, bottom = 0 }};

-- Settings
local fadeOutAlpha = 0;
local fadeInAlpha = 1;
local db = {
	-- Bar Frames
	["sActionBar"] = {
		["numbuttons"]	= 9,
		["scale"]		= 0.75,
		["size"]		= UI_BUTTON_SIZE,
		["spacing"]		= UI_BUTTON_PADDING,
		["anchors"]		= {"BOTTOM", 0, 139},
		["buttons"]		= "ActionButton",
		},
	["sABH_Right"] = {
		["hover"]		= 1,
		["scale"]		= 0.75,
		["anchors"]		= { "RIGHT", -(UI_BUTTON_PADDING * 2), 92 },
		["buttons"]		= "MultiBarRightButton",
		["size"]		= MultiBarBottomRightButton1:GetWidth(),
		["spacing"]		= -(UI_BUTTON_PADDING * 2),
		["numbuttons"]	= 7,
		},
	["sABH_Bottom"] = {
		["hover"]		= 1,
		["scale"]		= 0.75,
		["anchors"]		= { "BOTTOM", 0, 14.5 },
		["numbuttons"]	= 12,
		["size"]		= MultiBarBottomRightButton1:GetWidth(),
		["spacing"]		= UI_BUTTON_PADDING * 2,
		["buttons"]		= "MultiBarBottomRightButton",
		},
	["sABH_Bags"] = {
		["hide"]		= 1,
		["scale"]		= 0.9,
		["anchors"]		= { "TOPRIGHT", 0, 0 },
		["width"]		= 200,
		["height"]		= 60,
		["buttons"]		= { "MainMenuBarBackpackButton", "CharacterBag0Slot", "CharacterBag1Slot", "CharacterBag2Slot", "CharacterBag3Slot", "KeyRingButton" },
		},
	["sABH_MicroMenu"] = {
		["scale"]		= 0.8,
		["anchors"]		= { "BOTTOMLEFT", 2.5, 5.1 },
		["width"]		= 263,
		["height"]		= 48,
		["hover"]		= 1,
		["buttons"]		= { "CharacterMicroButton", "SpellbookMicroButton", "TalentMicroButton", "AchievementMicroButton", "QuestLogMicroButton", "GuildMicroButton", "PVPMicroButton", "LFDMicroButton", "EJMicroButton", "RaidMicroButton", "MainMenuMicroButton", "HelpMicroButton" },
		},
	["sABH_Pet"] = {
		["scale"]		= 1,
		["size"]		= UI_BUTTON_SIZE,
		["anchors"]		= { "BOTTOM", -312, 208 },
		["numbuttons"]	= NUM_PET_ACTION_SLOTS,
		["spacing"]		= UI_BUTTON_PADDING * 2,
		["buttons"]		= "PetActionButton",
		},
	["sABH_Shapeshift"] = {
		["scale"]		= 0.75,
		["size"]		= UI_BUTTON_SIZE,
		["spacing"]		= UI_BUTTON_PADDING,
		["anchors"]		= { "RIGHT", sActionBar, "LEFT", -(UI_BUTTON_PADDING), 0 },
		["numbuttons"]	= NUM_SHAPESHIFT_SLOTS,
		["buttons"]		= "ShapeshiftButton",
		},
	};

---------------------------------------------------
-- Create Action Bar Holder Frames
---------------------------------------------------

local ToggleBarVisibility = function(bar)
	if (bar:GetAlpha() ~= 0) then
		bar:SetAlpha(0);
		bar:SetScale(0.001);
	else
		bar:SetAlpha(1);
		bar:SetScale(db[bar:GetName()].scale);
	end
end

local SetBarAlpha = function(bar, alpha)
	if (bar:IsShown()) then
		for _, b in pairs(bar.buttons) do
			b:SetAlpha(alpha)
		end
	end
end

local LayoutActionBarHolder = function(frameName)
	local f = _G[frameName];

	if (db[frameName].size and db[frameName].spacing and #f.buttons > 0) then
		local w = db[frameName].size * #f.buttons + abs(db[frameName].spacing) * (#f.buttons - 1) + 0.5;
		local h = db[frameName].size;
		if (frameName == "sABH_Right") then
			f:SetWidth(h);
			f:SetHeight(w);
		else
			f:SetWidth(w);
			f:SetHeight(h);
		end
	elseif (db[frameName].width and db[frameName].height) then
		f:SetWidth(db[frameName].width);
		f:SetHeight(db[frameName].height);
	end
end

local SetupActionBarHolder = function(frameName)
	local f = _G[frameName] or CreateFrame("Frame", frameName, UIParent);
	f:SetParent(UIParent);
	f:ClearAllPoints();
	f:SetPoint(unpack(db[frameName].anchors));
	f:SetScale(db[frameName].scale);
	f:Show();

	-- Visibility
	if (db[frameName].hide == 1) then
		ToggleBarVisibility(f);
	end
	
	-- Store Buttons
	if (not f.buttons) then
		local i, b;
		f.buttons = {};
		if (type(db[frameName].buttons) == "string") then
			for i = 1, db[frameName].numbuttons or 12 do
				f.buttons[i] = _G[db[frameName].buttons..i];
			end
		else
			for i, b in pairs(db[frameName].buttons) do
				f.buttons[i] = _G[b];
			end
		end
	end

	-- Hide hidden buttons
	if (db[frameName].numbuttons and db[frameName].numbuttons < 12) then
		if (frameName == "sActionBar" or frameName == "sABH_Right" or frameName == "sABH_Bottom") then
			for i = db[frameName].numbuttons + 1, 12 do
				local b = _G[db[frameName].buttons..i];
				if (b) then
					b:ClearAllPoints();
					b:SetScale(0.001);
					b:SetAlpha(0);
					f.buttons[i] = nil;
				end
			end
		end
	end

	-- Resize
	LayoutActionBarHolder(frameName);
	
	-- Hover
	if (db[frameName].hover and db[frameName].hover == 1) then
		f:EnableMouse(true);
		f:SetScript("OnEnter", function(self) SetBarAlpha(f, fadeInAlpha); end);
		f:SetScript("OnLeave", function(self) SetBarAlpha(f, fadeOutAlpha); end);

		for _, b in pairs(f.buttons) do
			b:SetAlpha(fadeOutAlpha);
			b:HookScript("OnEnter", function(self) SetBarAlpha(f, fadeInAlpha); end);
			b:HookScript("OnLeave", function(self) SetBarAlpha(f, fadeOutAlpha); end);
		end
	end

	-- Debugging
	if (enableDebug == 1) then
		f:SetBackdrop(enableDebugBackDrop);
	end

	return f;
end

local sABH_Main = SetupActionBarHolder("sActionBar");				-- Main Action Bar
local sABH_Right = SetupActionBarHolder("sABH_Right");				-- Right Bar
local sABH_Bottom = SetupActionBarHolder("sABH_Bottom");			-- Bottom Bar
local sABH_Bags = SetupActionBarHolder("sABH_Bags");				-- Bags
local sABH_MicroMenu = SetupActionBarHolder("sABH_MicroMenu");		-- Micro Menu
local sABH_Pet = SetupActionBarHolder("sABH_Pet");					-- Pet Bar
local sABH_Shapeshift = SetupActionBarHolder("sABH_Shapeshift");	-- Shapeshift Bar

---------------------------------------------------
-- Move/Hide
---------------------------------------------------

local SetupActionButtons = function(bar, blizzardBar)
	local firstButton = true;
	local previousButton;
	local orientationH = (bar:GetWidth() >= bar:GetHeight());
	local frameName = bar:GetName();

	if (blizzardBar ~= nil) then
		blizzardBar:SetParent(bar);
		blizzardBar:ClearAllPoints();
		blizzardBar:SetPoint("TOPLEFT", 0, 0);
		blizzardBar:SetWidth(0.01);
	end
	
	for _, b in pairs(bar.buttons) do
		b:ClearAllPoints();
		
		if (db[frameName].size) then
			b:SetHeight(db[frameName].size);
			b:SetWidth(db[frameName].size);
		end

		if (firstButton) then
			b:SetPoint("TOPLEFT", bar.header or bar, "TOPLEFT");
			firstButton = false;
		else
			if (orientationH) then
				b:SetPoint("LEFT", previousButton, "RIGHT", db[frameName].spacing, 0);
			else
				b:SetPoint("TOP", previousButton, "BOTTOM", 0, db[frameName].spacing);
			end
		end
		previousButton = b;
	end
end

-- Layout Action Bar Buttons
SetupActionButtons(sABH_Main);
SetupActionButtons(sABH_Bottom, MultiBarBottomRight);
SetupActionButtons(sABH_Right, MultiBarRight);
SetupActionButtons(sABH_Shapeshift, ShapeshiftBarFrame);
SetupActionButtons(sABH_Pet, PetActionBarFrame);

-- Fix Shape Shift Bar Width
local LayoutShapeshiftBar = function()
	local numShapeshiftButtons = GetNumShapeshiftForms();
	for i = 1, NUM_SHAPESHIFT_SLOTS do
		if (i <= numShapeshiftButtons) then
			sABH_Shapeshift.buttons[i] = _G["ShapeshiftButton"..i];
		else
			sABH_Shapeshift.buttons[i] = nil;
		end
	end
	
	ShapeshiftButton1:ClearAllPoints();
	ShapeshiftButton1:SetPoint("TOPLEFT", sABH_Shapeshift, "TOPLEFT");
	LayoutActionBarHolder("sABH_Shapeshift");
end
hooksecurefunc("ShapeshiftBar_Update", LayoutShapeshiftBar);

-- Layout Bags
local LayoutBagsBar = function()
	for _, b in pairs(sABH_Bags.buttons) do
		b:SetParent(sABH_Bags);
	end

	MainMenuBarBackpackButton:ClearAllPoints();
	MainMenuBarBackpackButton:SetPoint("BOTTOMRIGHT", -15, 15);
end
LayoutBagsBar();

-- Layout Micro Menu
local function LayoutMicroMenu()
	for _, b in pairs(sABH_MicroMenu.buttons) do
		b:SetParent(sABH_MicroMenu);
	end

	CharacterMicroButton:ClearAllPoints();
	CharacterMicroButton:SetPoint("BOTTOMLEFT", 5, 5);
	UpdateMicroButtons();
end
hooksecurefunc("VehicleMenuBar_MoveMicroButtons", LayoutMicroMenu);  
LayoutMicroMenu();

---------------------------------------------------
-- Disable/Hide Blizzard Bars
---------------------------------------------------

UIPARENT_MANAGED_FRAME_POSITIONS["MultiBarBottomLeft"] = nil;
UIPARENT_MANAGED_FRAME_POSITIONS["MultiBarLeft"] = nil;
--BonusActionBarTexture0:Hide();
--BonusActionBarTexture1:Hide();
BonusActionBarFrame:UnregisterAllEvents();
BonusActionBarFrame:Hide();
MultiBarBottomLeft:Hide();
MultiBarLeft:Hide();
VehicleMenuBar:SetScale(0.001);
VehicleMenuBar:SetAlpha(0);
MainMenuBarArtFrame:UnregisterEvent("PLAYER_ENTERING_WORLD");
MainMenuBarArtFrame:UnregisterEvent("ACTIONBAR_PAGE_CHANGED");
MainMenuBarArtFrame:UnregisterEvent("ADDON_LOADED");
MainMenuBarArtFrame:UnregisterEvent("UNIT_ENTERING_VEHICLE");
MainMenuBarArtFrame:UnregisterEvent("UNIT_ENTERED_VEHICLE");
MainMenuBarArtFrame:UnregisterEvent("UNIT_EXITING_VEHICLE");
MainMenuBarArtFrame:UnregisterEvent("UNIT_EXITED_VEHICLE");
MainMenuBarArtFrame:Hide();
MainMenuBar:UnregisterAllEvents();
MainMenuBar:SetScale(0.001);
MainMenuBar:SetAlpha(0);
MainMenuExpBar:UnregisterAllEvents();
MainMenuExpBar:Hide();
PossessBarFrame:UnregisterAllEvents();
PossessBarFrame:Hide();

---------------------------------------------------
-- Fixes
---------------------------------------------------

-- Fix Bag Positions
UIPARENT_MANAGED_FRAME_POSITIONS["CONTAINER_OFFSET_X"] = nil;
UIPARENT_MANAGED_FRAME_POSITIONS["CONTAINER_OFFSET_Y"] = nil;

-- Fix Micro Menu Alpha
if (db["sABH_MicroMenu"].hover and db["sABH_MicroMenu"].hover == 1) then
	local UpdateMicroButtons_Hover = function()
		if (not sABH_MicroMenu:IsMouseOver(1, -1, -1, 1)) then
			SetBarAlpha(sABH_MicroMenu, fadeOutAlpha);
		end
	end
	
	hooksecurefunc("UpdateMicroButtons", UpdateMicroButtons_Hover);
end

-- Command: /BAGS
SLASH_BAGS1 = "/bags";
SlashCmdList["BAGS"] = function() ToggleBarVisibility(sABH_Bags); end;

-- Update Options
local UpdateActionBarOptions = function(self, event)
	SHOW_MULTI_ACTIONBAR_1 = nil;
	SHOW_MULTI_ACTIONBAR_2 = 1;
	SHOW_MULTI_ACTIONBAR_3 = 1;
	SHOW_MULTI_ACTIONBAR_4 = nil;
	InterfaceOptionsActionBarsPanelBottomLeft.value		= SHOW_MULTI_ACTIONBAR_1 or 0;
	InterfaceOptionsActionBarsPanelBottomRight.value	= SHOW_MULTI_ACTIONBAR_2 or 0;
	InterfaceOptionsActionBarsPanelRight.value			= SHOW_MULTI_ACTIONBAR_3 or 0;
	InterfaceOptionsActionBarsPanelRightTwo.value		= SHOW_MULTI_ACTIONBAR_4 or 0;
	SetActionBarToggles(SHOW_MULTI_ACTIONBAR_1, SHOW_MULTI_ACTIONBAR_2, SHOW_MULTI_ACTIONBAR_3, SHOW_MULTI_ACTIONBAR_4, ALWAYS_SHOW_MULTIBARS);
	MultiActionBar_Update();
	UIParent_ManageFramePositions();
	
	self:UnregisterAllEvents();
end

local sAB_SettingsUpdater = CreateFrame("Frame");
sAB_SettingsUpdater:RegisterEvent("PLAYER_ENTERING_WORLD");
sAB_SettingsUpdater:SetScript("OnEvent", UpdateActionBarOptions);
